@@ -1,6 +1,6 @@
 /*
  * sar, sadc, sadf, mpstat and iostat common routines.
- * (C) 1999-2019 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2020 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -189,6 +189,7 @@ void init_nls(void)
  * ioconf.c which should be used only with kernels that don't have sysfs.
  *
  * IN:
+ * @sysdev		sysfs location.
  * @name		Device or partition name.
  * @allow_virtual	TRUE if virtual devices are also accepted.
  *			The device is assumed to be virtual if no
@@ -198,7 +199,7 @@ void init_nls(void)
  * TRUE if @name is not a partition.
  ***************************************************************************
  */
-int is_device(char *name, int allow_virtual)
+int is_device(char *sysdev, char *name, int allow_virtual)
 {
 	char syspath[PATH_MAX];
 	char *slash;
@@ -207,7 +208,7 @@ int is_device(char *name, int allow_virtual)
 	while ((slash = strchr(name, '/'))) {
 		*slash = '!';
 	}
-	snprintf(syspath, sizeof(syspath), "%s/%s%s", SYSFS_BLOCK, name,
+	snprintf(syspath, sizeof(syspath), "%s/%s/%s%s", sysdev, __BLOCK, name,
 		 allow_virtual ? "" : "/device");
 
 	return !(access(syspath, F_OK));
@@ -392,8 +393,8 @@ int get_wwnid_from_pretty(char *pretty, unsigned long long *wwn, unsigned int *p
 
 		if (!strncmp(name, pretty, FILENAME_MAX)) {
 			/* We have found pretty name for current persistent one */
-			strncpy(wwn_name, drd->d_name, FILENAME_MAX);
-			wwn_name[FILENAME_MAX - 1] = '\0';
+			strncpy(wwn_name, drd->d_name, MINIMUM(sizeof(wwn_name), sizeof(drd->d_name)));
+			wwn_name[sizeof(wwn_name) - 1] = '\0';
 
 			/* Try to extract WWN */
 			if (!extract_wwnid(wwn_name, wwn, part_nr)) {
@@ -410,6 +411,28 @@ int get_wwnid_from_pretty(char *pretty, unsigned long long *wwn, unsigned int *p
 	return rc;
 }
 
+/*
+ ***************************************************************************
+ * Check if a directory exists.
+ *
+ * IN:
+ * @dirname	Name of the directory.
+ *
+ * RETURNS:
+ * TRUE if @dirname is actually an existing directory.
+ ***************************************************************************
+ */
+int check_dir(char *dirname)
+{
+	struct stat sb;
+
+	if (!stat(dirname, &sb) && S_ISDIR(sb.st_mode))
+		return 1;
+
+	return 0;
+}
+
+
 #ifndef SOURCE_SADC
 /*
  ***************************************************************************
@@ -442,52 +465,6 @@ int count_csvalues(int arg_c, char **arg_v)
 	return nr;
 }
 
-/*
- ***************************************************************************
- * Look for partitions of a given block device in /sys filesystem.
- *
- * IN:
- * @dev_name	Name of the block device.
- *
- * RETURNS:
- * Number of partitions for the given block device.
- ***************************************************************************
- */
-int get_dev_part_nr(char *dev_name)
-{
-	DIR *dir;
-	struct dirent *drd;
-	char dfile[MAX_PF_NAME], line[MAX_PF_NAME];
-	int part = 0, err;
-
-	snprintf(dfile, MAX_PF_NAME, "%s/%s", SYSFS_BLOCK, dev_name);
-	dfile[MAX_PF_NAME - 1] = '\0';
-
-	/* Open current device directory in /sys/block */
-	if ((dir = opendir(dfile)) == NULL)
-		return 0;
-
-	/* Get current file entry */
-	while ((drd = readdir(dir)) != NULL) {
-		if (!strcmp(drd->d_name, ".") || !strcmp(drd->d_name, ".."))
-			continue;
-		err = snprintf(line, MAX_PF_NAME, "%s/%s/%s", dfile, drd->d_name, S_STAT);
-		if ((err < 0) || (err >= MAX_PF_NAME))
-			continue;
-
-		/* Try to guess if current entry is a directory containing a stat file */
-		if (!access(line, R_OK)) {
-			/* Yep... */
-			part++;
-		}
-	}
-
-	/* Close directory */
-	closedir(dir);
-
-	return part;
-}
-
 /*
  ***************************************************************************
  * Read /proc/devices file and get device-mapper major number.
@@ -746,8 +723,8 @@ char *device_name(char *name)
 	if (!strncmp(resolved_name, "/dev/", 5)) {
 		i = 5;
 	}
-	strncpy(out, resolved_name + i, MAX_FILE_LEN);
-	out[MAX_FILE_LEN - 1] = '\0';
+	strncpy(out, resolved_name + i, sizeof(out));
+	out[sizeof(out) - 1] = '\0';
 
 	/* Some devices may have a slash in their name (eg. cciss/c0d0...) */
 	while ((slash = strchr(out, '/'))) {
@@ -865,16 +842,18 @@ char *strtolower(char *str)
  * @type	Persistent type name (UUID, LABEL, etc.)
  *
  * RETURNS:
- * Path to the persistent type name directory, or NULL if access is denied.
+ * Path to the persistent type name directory, or NULL if access is denied
+ * or strings have been truncated.
  ***************************************************************************
 */
 char *get_persistent_type_dir(char *type)
 {
 	static char dir[PATH_MAX];
+	int n;
 
-	snprintf(dir, sizeof(dir), "%s-%s", DEV_DISK_BY, type);
+	n = snprintf(dir, sizeof(dir), "%s-%s", DEV_DISK_BY, type);
 
-	if (access(dir, R_OK)) {
+	if ((n >= sizeof(dir)) || access(dir, R_OK)) {
 		return (NULL);
 	}
 
@@ -889,17 +868,19 @@ char *get_persistent_type_dir(char *type)
  * @name	Persistent name.
  *
  * RETURNS:
- * Path to the persistent name, or NULL if file doesn't exist.
+ * Path to the persistent name, or NULL if file doesn't exist or strings
+ * have been truncated.
  ***************************************************************************
 */
 char *get_persistent_name_path(char *name)
 {
 	static char path[PATH_MAX];
+	int n;
 
-	snprintf(path, PATH_MAX, "%s/%s",
-		 get_persistent_type_dir(persistent_name_type), name);
+	n = snprintf(path, sizeof(path), "%s/%s",
+		     get_persistent_type_dir(persistent_name_type), name);
 
-	if (access(path, F_OK)) {
+	if ((n >= sizeof(path)) || access(path, F_OK)) {
 		return (NULL);
 	}
 
@@ -1015,8 +996,8 @@ char *get_persistent_name_from_pretty(char *pretty)
 
 		if (!strncmp(name, pretty, FILENAME_MAX)) {
 			/* We have found pretty name for current persistent one */
-			strncpy(persist_name, persist_names[i], FILENAME_MAX);
-			persist_name[FILENAME_MAX - 1] = '\0';
+			strncpy(persist_name, persist_names[i], sizeof(persist_name));
+			persist_name[sizeof(persist_name) - 1] = '\0';
 			break;
 		}
 	}
