@@ -1,6 +1,6 @@
 /*
  * json_stats.c: Funtions used by sadf to display statistics in JSON format.
- * (C) 1999-2019 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2020 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -93,6 +93,34 @@ void json_markup_power_management(int tab, int action)
 	}
 }
 
+/*
+ ***************************************************************************
+ * Open or close "psi" markup.
+ *
+ * IN:
+ * @tab		Number of tabulations.
+ * @action	Open or close action.
+ ***************************************************************************
+ */
+void json_markup_psi(int tab, int action)
+{
+	static int markup_state = CLOSE_JSON_MARKUP;
+
+	if (action == markup_state)
+		return;
+	markup_state = action;
+
+	if (action == OPEN_JSON_MARKUP) {
+		/* Open markup */
+		xprintf(tab, "\"psi\": {");
+	}
+	else {
+		/* Close markup */
+		printf("\n");
+		xprintf0(tab, "}");
+	}
+}
+
 /*
  ***************************************************************************
  * Display CPU statistics in JSON.
@@ -809,7 +837,7 @@ __print_funct_t json_print_net_dev_stats(struct activity *a, int curr, int tab,
 
 	memset(&sndzero, 0, STATS_NET_DEV_SIZE);
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -895,7 +923,7 @@ __print_funct_t json_print_net_edev_stats(struct activity *a, int curr, int tab,
 	struct stats_net_edev *snedc, *snedp, snedzero;
 	int sep = FALSE;
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	memset(&snedzero, 0, STATS_NET_EDEV_SIZE);
@@ -981,7 +1009,7 @@ __print_funct_t json_print_net_nfs_stats(struct activity *a, int curr, int tab,
 		*snnc = (struct stats_net_nfs *) a->buf[curr],
 		*snnp = (struct stats_net_nfs *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -1026,7 +1054,7 @@ __print_funct_t json_print_net_nfsd_stats(struct activity *a, int curr, int tab,
 		*snndc = (struct stats_net_nfsd *) a->buf[curr],
 		*snndp = (struct stats_net_nfsd *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -1080,7 +1108,7 @@ __print_funct_t json_print_net_sock_stats(struct activity *a, int curr, int tab,
 	struct stats_net_sock
 		*snsc = (struct stats_net_sock *) a->buf[curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -1125,7 +1153,7 @@ __print_funct_t json_print_net_ip_stats(struct activity *a, int curr, int tab,
 		*snic = (struct stats_net_ip *) a->buf[curr],
 		*snip = (struct stats_net_ip *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -1174,7 +1202,7 @@ __print_funct_t json_print_net_eip_stats(struct activity *a, int curr, int tab,
 		*sneic = (struct stats_net_eip *) a->buf[curr],
 		*sneip = (struct stats_net_eip *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -1223,7 +1251,7 @@ __print_funct_t json_print_net_icmp_stats(struct activity *a, int curr, int tab,
 		*snic = (struct stats_net_icmp *) a->buf[curr],
 		*snip = (struct stats_net_icmp *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -1284,7 +1312,7 @@ __print_funct_t json_print_net_eicmp_stats(struct activity *a, int curr, int tab
 		*sneic = (struct stats_net_eicmp *) a->buf[curr],
 		*sneip = (struct stats_net_eicmp *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -1341,7 +1369,7 @@ __print_funct_t json_print_net_tcp_stats(struct activity *a, int curr, int tab,
 		*sntc = (struct stats_net_tcp *) a->buf[curr],
 		*sntp = (struct stats_net_tcp *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -1382,7 +1410,7 @@ __print_funct_t json_print_net_etcp_stats(struct activity *a, int curr, int tab,
 		*snetc = (struct stats_net_etcp *) a->buf[curr],
 		*snetp = (struct stats_net_etcp *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -1425,7 +1453,7 @@ __print_funct_t json_print_net_udp_stats(struct activity *a, int curr, int tab,
 		*snuc = (struct stats_net_udp *) a->buf[curr],
 		*snup = (struct stats_net_udp *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -1465,7 +1493,7 @@ __print_funct_t json_print_net_sock6_stats(struct activity *a, int curr, int tab
 	struct stats_net_sock6
 		*snsc = (struct stats_net_sock6 *) a->buf[curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -1506,7 +1534,7 @@ __print_funct_t json_print_net_ip6_stats(struct activity *a, int curr, int tab,
 		*snic = (struct stats_net_ip6 *) a->buf[curr],
 		*snip = (struct stats_net_ip6 *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -1559,7 +1587,7 @@ __print_funct_t json_print_net_eip6_stats(struct activity *a, int curr, int tab,
 		*sneic = (struct stats_net_eip6 *) a->buf[curr],
 		*sneip = (struct stats_net_eip6 *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -1614,7 +1642,7 @@ __print_funct_t json_print_net_icmp6_stats(struct activity *a, int curr, int tab
 		*snic = (struct stats_net_icmp6 *) a->buf[curr],
 		*snip = (struct stats_net_icmp6 *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -1681,7 +1709,7 @@ __print_funct_t json_print_net_eicmp6_stats(struct activity *a, int curr, int ta
 		*sneic = (struct stats_net_eicmp6 *) a->buf[curr],
 		*sneip = (struct stats_net_eicmp6 *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -1736,7 +1764,7 @@ __print_funct_t json_print_net_udp6_stats(struct activity *a, int curr, int tab,
 		*snuc = (struct stats_net_udp6 *) a->buf[curr],
 		*snup = (struct stats_net_udp6 *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_json_markup;
 
 	json_markup_network(tab, OPEN_JSON_MARKUP);
@@ -2409,3 +2437,142 @@ close_json_markup:
 		json_markup_network(tab, CLOSE_JSON_MARKUP);
 	}
 }
+
+/*
+ ***************************************************************************
+ * Display pressure-stall CPU statistics in JSON.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @curr	Index in array for current sample statistics.
+ * @tab		Indentation in output.
+ * @itv		Interval of time in 1/100th of a second.
+ ***************************************************************************
+ */
+__print_funct_t json_print_psicpu_stats(struct activity *a, int curr, int tab,
+				        unsigned long long itv)
+{
+	struct stats_psi_cpu
+		*psic = (struct stats_psi_cpu *) a->buf[curr],
+		*psip = (struct stats_psi_cpu *) a->buf[!curr];
+
+	if (!IS_SELECTED(a->options))
+		goto close_json_markup;
+
+	json_markup_psi(tab, OPEN_JSON_MARKUP);
+	tab++;
+
+	xprintf0(tab, "\"psi-cpu\": {"
+		 "\"some_avg10\": %.2f, "
+		 "\"some_avg60\": %.2f, "
+		 "\"some_avg300\": %.2f, "
+		 "\"some_avg\": %.2f}",
+		 (double) psic->some_acpu_10  / 100,
+		 (double) psic->some_acpu_60  / 100,
+		 (double) psic->some_acpu_300 / 100,
+		 ((double) psic->some_cpu_total - psip->some_cpu_total) / (100 * itv));
+	tab--;
+
+close_json_markup:
+	if (CLOSE_MARKUP(a->options)) {
+		json_markup_psi(tab, CLOSE_JSON_MARKUP);
+	}
+}
+
+/*
+ ***************************************************************************
+ * Display pressure-stall I/O statistics in JSON.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @curr	Index in array for current sample statistics.
+ * @tab		Indentation in output.
+ * @itv		Interval of time in 1/100th of a second.
+ ***************************************************************************
+ */
+__print_funct_t json_print_psiio_stats(struct activity *a, int curr, int tab,
+				       unsigned long long itv)
+{
+	struct stats_psi_io
+		*psic = (struct stats_psi_io *) a->buf[curr],
+		*psip = (struct stats_psi_io *) a->buf[!curr];
+
+	if (!IS_SELECTED(a->options))
+		goto close_json_markup;
+
+	json_markup_psi(tab, OPEN_JSON_MARKUP);
+	tab++;
+
+	xprintf0(tab, "\"psi-io\": {"
+		 "\"some_avg10\": %.2f, "
+		 "\"some_avg60\": %.2f, "
+		 "\"some_avg300\": %.2f, "
+		 "\"some_avg\": %.2f, "
+		 "\"full_avg10\": %.2f, "
+		 "\"full_avg60\": %.2f, "
+		 "\"full_avg300\": %.2f, "
+		 "\"full_avg\": %.2f}",
+		 (double) psic->some_aio_10  / 100,
+		 (double) psic->some_aio_60  / 100,
+		 (double) psic->some_aio_300 / 100,
+		 ((double) psic->some_io_total - psip->some_io_total) / (100 * itv),
+		 (double) psic->full_aio_10  / 100,
+		 (double) psic->full_aio_60  / 100,
+		 (double) psic->full_aio_300 / 100,
+		 ((double) psic->full_io_total - psip->full_io_total) / (100 * itv));
+	tab--;
+
+close_json_markup:
+	if (CLOSE_MARKUP(a->options)) {
+		json_markup_psi(tab, CLOSE_JSON_MARKUP);
+	}
+}
+
+/*
+ ***************************************************************************
+ * Display pressure-stall memory statistics in JSON.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @curr	Index in array for current sample statistics.
+ * @tab		Indentation in output.
+ * @itv		Interval of time in 1/100th of a second.
+ ***************************************************************************
+ */
+__print_funct_t json_print_psimem_stats(struct activity *a, int curr, int tab,
+				        unsigned long long itv)
+{
+	struct stats_psi_mem
+		*psic = (struct stats_psi_mem *) a->buf[curr],
+		*psip = (struct stats_psi_mem *) a->buf[!curr];
+
+	if (!IS_SELECTED(a->options))
+		goto close_json_markup;
+
+	json_markup_psi(tab, OPEN_JSON_MARKUP);
+	tab++;
+
+	xprintf0(tab, "\"psi-mem\": {"
+		 "\"some_avg10\": %.2f, "
+		 "\"some_avg60\": %.2f, "
+		 "\"some_avg300\": %.2f, "
+		 "\"some_avg\": %.2f, "
+		 "\"full_avg10\": %.2f, "
+		 "\"full_avg60\": %.2f, "
+		 "\"full_avg300\": %.2f, "
+		 "\"full_avg\": %.2f}",
+		 (double) psic->some_amem_10  / 100,
+		 (double) psic->some_amem_60  / 100,
+		 (double) psic->some_amem_300 / 100,
+		 ((double) psic->some_mem_total - psip->some_mem_total) / (100 * itv),
+		 (double) psic->full_amem_10  / 100,
+		 (double) psic->full_amem_60  / 100,
+		 (double) psic->full_amem_300 / 100,
+		 ((double) psic->full_mem_total - psip->full_mem_total) / (100 * itv));
+	tab--;
+
+close_json_markup:
+	if (CLOSE_MARKUP(a->options)) {
+		json_markup_psi(tab, CLOSE_JSON_MARKUP);
+	}
+}
