@@ -1,6 +1,6 @@
 /*
  * raw_stats.c: Functions used by sar to display statistics in raw format.
- * (C) 1999-2019 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2020 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -53,8 +53,8 @@ char *pfield(char *hdr_line, int pos)
 	int i, j = 0;
 
 	if (hdr_line) {
-		strncpy(hline, hdr_line, HEADER_LINE_LEN - 1);
-		hline[HEADER_LINE_LEN - 1] = '\0';
+		strncpy(hline, hdr_line, sizeof(hline) - 1);
+		hline[sizeof(hline) - 1] = '\0';
 		idx = 0;
 
 		for (hl = strtok(hline, "|"); hl && (pos > 0); hl = strtok(NULL, "|"), pos--);
@@ -67,8 +67,8 @@ char *pfield(char *hdr_line, int pos)
 			j = strcspn(hl, "&");
 			*(hl + j) = ';';
 		}
-		strncpy(field, hl, HEADER_LINE_LEN);
-		field[HEADER_LINE_LEN - 1] = '\0';
+		strncpy(field, hl, sizeof(field));
+		field[sizeof(field) - 1] = '\0';
 	}
 
 	/* Display current field */
@@ -1664,3 +1664,87 @@ __print_funct_t raw_print_softnet_stats(struct activity *a, char *timestr, int c
 		printf("\n");
 	}
 }
+
+/*
+ ***************************************************************************
+ * Display pressure-stall CPU statistics in raw format.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @timestr	Time for current statistics sample.
+ * @curr	Index in array for current sample statistics.
+ ***************************************************************************
+ */
+__print_funct_t raw_print_psicpu_stats(struct activity *a, char *timestr, int curr)
+{
+	struct stats_psi_cpu
+		*psic = (struct stats_psi_cpu *) a->buf[curr],
+		*psip = (struct stats_psi_cpu *) a->buf[!curr];
+
+	printf("%s; %s; %lu;", timestr, pfield(a->hdr_line, FIRST), psic->some_acpu_10);
+	printf(" %s; %lu;", pfield(NULL, 0), psic->some_acpu_60);
+	printf(" %s; %lu;", pfield(NULL, 0), psic->some_acpu_300);
+	printf(" %s", pfield(NULL, 0));
+	pval((unsigned long long) psip->some_cpu_total, (unsigned long long) psic->some_cpu_total);
+	printf("\n");
+}
+
+/*
+ ***************************************************************************
+ * Display pressure-stall I/O statistics in raw format.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @timestr	Time for current statistics sample.
+ * @curr	Index in array for current sample statistics.
+ ***************************************************************************
+ */
+__print_funct_t raw_print_psiio_stats(struct activity *a, char *timestr, int curr)
+{
+	struct stats_psi_io
+		*psic = (struct stats_psi_io *) a->buf[curr],
+		*psip = (struct stats_psi_io *) a->buf[!curr];
+
+	printf("%s; %s; %lu;", timestr, pfield(a->hdr_line, FIRST), psic->some_aio_10);
+	printf(" %s; %lu;", pfield(NULL, 0), psic->some_aio_60);
+	printf(" %s; %lu;", pfield(NULL, 0), psic->some_aio_300);
+	printf(" %s", pfield(NULL, 0));
+	pval((unsigned long long) psip->some_io_total, (unsigned long long) psic->some_io_total);
+
+	printf(" %s; %lu;", pfield(NULL, 0), psic->full_aio_10);
+	printf(" %s; %lu;", pfield(NULL, 0), psic->full_aio_60);
+	printf(" %s; %lu;", pfield(NULL, 0), psic->full_aio_300);
+	printf(" %s", pfield(NULL, 0));
+	pval((unsigned long long) psip->full_io_total, (unsigned long long) psic->full_io_total);
+	printf("\n");
+}
+
+/*
+ ***************************************************************************
+ * Display pressure-stall mem statistics in raw format.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @timestr	Time for current statistics sample.
+ * @curr	Index in array for current sample statistics.
+ ***************************************************************************
+ */
+__print_funct_t raw_print_psimem_stats(struct activity *a, char *timestr, int curr)
+{
+	struct stats_psi_mem
+		*psic = (struct stats_psi_mem *) a->buf[curr],
+		*psip = (struct stats_psi_mem *) a->buf[!curr];
+
+	printf("%s; %s; %lu;", timestr, pfield(a->hdr_line, FIRST), psic->some_amem_10);
+	printf(" %s; %lu;", pfield(NULL, 0), psic->some_amem_60);
+	printf(" %s; %lu;", pfield(NULL, 0), psic->some_amem_300);
+	printf(" %s", pfield(NULL, 0));
+	pval((unsigned long long) psip->some_mem_total, (unsigned long long) psic->some_mem_total);
+
+	printf(" %s; %lu;", pfield(NULL, 0), psic->full_amem_10);
+	printf(" %s; %lu;", pfield(NULL, 0), psic->full_amem_60);
+	printf(" %s; %lu;", pfield(NULL, 0), psic->full_amem_300);
+	printf(" %s", pfield(NULL, 0));
+	pval((unsigned long long) psip->full_mem_total, (unsigned long long) psic->full_mem_total);
+	printf("\n");
+}
