@@ -1,6 +1,6 @@
 /*
  * rd_stats.h: Include file used to read system statistics
- * (C) 1999-2019 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2020 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _RD_STATS_H
@@ -65,6 +65,10 @@
 #define NET_RPC_NFSD	PRE "/proc/net/rpc/nfsd"
 #define NET_SOFTNET	PRE "/proc/net/softnet_stat"
 #define LOADAVG		PRE "/proc/loadavg"
+#define PRESSURE	PRE "/proc/pressure"
+#define PSI_CPU		PRESSURE "/cpu"
+#define PSI_IO		PRESSURE "/io"
+#define PSI_MEM		PRESSURE "/memory"
 #define VMSTAT		PRE "/proc/vmstat"
 #define NET_SNMP	PRE "/proc/net/snmp"
 #define NET_SNMP6	PRE "/proc/net/snmp6"
@@ -262,6 +266,7 @@ struct stats_serial {
 /* Structure for block devices statistics */
 struct stats_disk {
 	unsigned long long nr_ios;
+	unsigned long long wwn[2];
 	unsigned long	   rd_sect	__attribute__ ((aligned (8)));
 	unsigned long	   wr_sect	__attribute__ ((aligned (8)));
 	unsigned long	   dc_sect	__attribute__ ((aligned (8)));
@@ -272,12 +277,13 @@ struct stats_disk {
 	unsigned int	   major;
 	unsigned int	   minor;
 	unsigned int	   dc_ticks;
+	unsigned int	   part_nr;
 };
 
 #define STATS_DISK_SIZE	(sizeof(struct stats_disk))
-#define STATS_DISK_ULL	1
+#define STATS_DISK_ULL	3
 #define STATS_DISK_UL	3
-#define STATS_DISK_U	7
+#define STATS_DISK_U	8
 
 /* Structure for network interfaces statistics */
 struct stats_net_dev {
@@ -702,6 +708,53 @@ struct stats_softnet {
 #define STATS_SOFTNET_UL	0
 #define STATS_SOFTNET_U		5
 
+/* Structure for pressure-stall CPU statistics */
+struct stats_psi_cpu {
+	unsigned long long some_cpu_total;
+	unsigned long	   some_acpu_10;
+	unsigned long	   some_acpu_60;
+	unsigned long	   some_acpu_300;
+};
+
+#define STATS_PSI_CPU_SIZE	(sizeof(struct stats_psi_cpu))
+#define STATS_PSI_CPU_ULL	1
+#define STATS_PSI_CPU_UL	3
+#define STATS_PSI_CPU_U		0
+
+/* Structure for pressure-stall I/O statistics */
+struct stats_psi_io {
+	unsigned long long some_io_total;
+	unsigned long long full_io_total;
+	unsigned long	   some_aio_10;
+	unsigned long	   some_aio_60;
+	unsigned long	   some_aio_300;
+	unsigned long	   full_aio_10;
+	unsigned long	   full_aio_60;
+	unsigned long	   full_aio_300;
+};
+
+#define STATS_PSI_IO_SIZE	(sizeof(struct stats_psi_io))
+#define STATS_PSI_IO_ULL	2
+#define STATS_PSI_IO_UL		6
+#define STATS_PSI_IO_U		0
+
+/* Structure for pressure-stall memory statistics */
+struct stats_psi_mem {
+	unsigned long long some_mem_total;
+	unsigned long long full_mem_total;
+	unsigned long	   some_amem_10;
+	unsigned long	   some_amem_60;
+	unsigned long	   some_amem_300;
+	unsigned long	   full_amem_10;
+	unsigned long	   full_amem_60;
+	unsigned long	   full_amem_300;
+};
+
+#define STATS_PSI_MEM_SIZE	(sizeof(struct stats_psi_mem))
+#define STATS_PSI_MEM_ULL	2
+#define STATS_PSI_MEM_UL	6
+#define STATS_PSI_MEM_U		0
+
 /*
  ***************************************************************************
  * Prototypes for functions used to read system statistics
@@ -792,6 +845,13 @@ __nr_t read_fchost
 	(struct stats_fchost *, __nr_t);
 int read_softnet
 	(struct stats_softnet *, __nr_t, unsigned char []);
+__nr_t read_psicpu
+	(struct stats_psi_cpu *);
+__nr_t read_psiio
+	(struct stats_psi_io *);
+__nr_t read_psimem
+	(struct stats_psi_mem *);
+
 #endif /* SOURCE_SADC */
 
 #endif /* _RD_STATS_H */
