@@ -1,6 +1,6 @@
 /*
  * rndr_stats.c: Funtions used by sadf to display statistics in selected format.
- * (C) 1999-2019 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2020 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -3193,3 +3193,189 @@ __print_funct_t render_softnet_stats(struct activity *a, int isdb, char *pre,
 		}
 	}
 }
+
+/*
+ ***************************************************************************
+ * Display pressure-stall CPU statistics in selected format.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @isdb	Flag, true if db printing, false if ppc printing.
+ * @pre		Prefix string for output entries
+ * @curr	Index in array for current sample statistics.
+ * @itv		Interval of time in 1/100th of a second.
+ ***************************************************************************
+ */
+__print_funct_t render_psicpu_stats(struct activity *a, int isdb, char *pre,
+				    int curr, unsigned long long itv)
+{
+	struct stats_psi_cpu
+		*psic = (struct stats_psi_cpu *) a->buf[curr],
+		*psip = (struct stats_psi_cpu *) a->buf[!curr];
+	int pt_newlin
+		= (DISPLAY_HORIZONTALLY(flags) ? PT_NOFLAG : PT_NEWLIN);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%scpu-10", NULL, NULL,
+	       NOVAL,
+	       (double) psic->some_acpu_10 / 100,
+	       NULL);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%scpu-60", NULL, NULL,
+	       NOVAL,
+	       (double) psic->some_acpu_60 / 100,
+	       NULL);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%scpu-300", NULL, NULL,
+	       NOVAL,
+	       (double) psic->some_acpu_300 / 100,
+	       NULL);
+
+	render(isdb, pre, pt_newlin,
+	       "-\t%scpu", NULL, NULL,
+	       NOVAL,
+	       ((double) psic->some_cpu_total - psip->some_cpu_total) / (100 * itv),
+	       NULL);
+}
+
+/*
+ ***************************************************************************
+ * Display pressure-stall I/O statistics in selected format.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @isdb	Flag, true if db printing, false if ppc printing.
+ * @pre		Prefix string for output entries
+ * @curr	Index in array for current sample statistics.
+ * @itv		Interval of time in 1/100th of a second.
+ ***************************************************************************
+ */
+__print_funct_t render_psiio_stats(struct activity *a, int isdb, char *pre,
+				   int curr, unsigned long long itv)
+{
+	struct stats_psi_io
+		*psic = (struct stats_psi_io *) a->buf[curr],
+		*psip = (struct stats_psi_io *) a->buf[!curr];
+	int pt_newlin
+		= (DISPLAY_HORIZONTALLY(flags) ? PT_NOFLAG : PT_NEWLIN);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%sio-10", NULL, NULL,
+	       NOVAL,
+	       (double) psic->some_aio_10 / 100,
+	       NULL);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%sio-60", NULL, NULL,
+	       NOVAL,
+	       (double) psic->some_aio_60 / 100,
+	       NULL);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%sio-300", NULL, NULL,
+	       NOVAL,
+	       (double) psic->some_aio_300 / 100,
+	       NULL);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%sio", NULL, NULL,
+	       NOVAL,
+	       ((double) psic->some_io_total - psip->some_io_total) / (100 * itv),
+	       NULL);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%fio-10", NULL, NULL,
+	       NOVAL,
+	       (double) psic->full_aio_10 / 100,
+	       NULL);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%fio-60", NULL, NULL,
+	       NOVAL,
+	       (double) psic->full_aio_60 / 100,
+	       NULL);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%fio-300", NULL, NULL,
+	       NOVAL,
+	       (double) psic->full_aio_300 / 100,
+	       NULL);
+
+	render(isdb, pre, pt_newlin,
+	       "-\t%fio", NULL, NULL,
+	       NOVAL,
+	       ((double) psic->full_io_total - psip->full_io_total) / (100 * itv),
+	       NULL);
+}
+
+/*
+ ***************************************************************************
+ * Display pressure-stall memory statistics in selected format.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @isdb	Flag, true if db printing, false if ppc printing.
+ * @pre		Prefix string for output entries
+ * @curr	Index in array for current sample statistics.
+ * @itv		Interval of time in 1/100th of a second.
+ ***************************************************************************
+ */
+__print_funct_t render_psimem_stats(struct activity *a, int isdb, char *pre,
+				    int curr, unsigned long long itv)
+{
+	struct stats_psi_mem
+		*psic = (struct stats_psi_mem *) a->buf[curr],
+		*psip = (struct stats_psi_mem *) a->buf[!curr];
+	int pt_newlin
+		= (DISPLAY_HORIZONTALLY(flags) ? PT_NOFLAG : PT_NEWLIN);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%smem-10", NULL, NULL,
+	       NOVAL,
+	       (double) psic->some_amem_10 / 100,
+	       NULL);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%smem-60", NULL, NULL,
+	       NOVAL,
+	       (double) psic->some_amem_60 / 100,
+	       NULL);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%smem-300", NULL, NULL,
+	       NOVAL,
+	       (double) psic->some_amem_300 / 100,
+	       NULL);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%smem", NULL, NULL,
+	       NOVAL,
+	       ((double) psic->some_mem_total - psip->some_mem_total) / (100 * itv),
+	       NULL);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%fmem-10", NULL, NULL,
+	       NOVAL,
+	       (double) psic->full_amem_10 / 100,
+	       NULL);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%fmem-60", NULL, NULL,
+	       NOVAL,
+	       (double) psic->full_amem_60 / 100,
+	       NULL);
+
+	render(isdb, pre, PT_NOFLAG,
+	       "-\t%fmem-300", NULL, NULL,
+	       NOVAL,
+	       (double) psic->full_amem_300 / 100,
+	       NULL);
+
+	render(isdb, pre, pt_newlin,
+	       "-\t%fmem", NULL, NULL,
+	       NOVAL,
+	       ((double) psic->full_mem_total - psip->full_mem_total) / (100 * itv),
+	       NULL);
+}
