@@ -1,6 +1,6 @@
 /*
  * sar/sadc: Report system activity
- * (C) 1999-2019 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2020 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _SA_H
@@ -20,12 +20,12 @@
  */
 
 /* Number of activities */
-#define NR_ACT		39
+#define NR_ACT		42
 /* The value below is used for sanity check */
 #define MAX_NR_ACT	256
 
 /* Number of functions used to count items */
-#define NR_F_COUNT	11
+#define NR_F_COUNT	12
 
 /* Activities */
 #define A_CPU		1
@@ -67,6 +67,9 @@
 #define A_FS		37
 #define A_NET_FC	38
 #define A_NET_SOFT	39
+#define A_PSI_CPU	40
+#define A_PSI_IO	41
+#define A_PSI_MEM	42
 
 
 /* Macro used to flag an activity that should be collected */
@@ -189,6 +192,7 @@
 /* Keywords */
 #define K_A_NULL	"A_NULL"
 #define K_CPU		"CPU"
+#define K_PSI_CPU	"CPU"
 #define K_DEV		"DEV"
 #define K_EDEV		"EDEV"
 #define K_EICMP		"EICMP"
@@ -202,11 +206,15 @@
 #define K_ICMP		"ICMP"
 #define K_ICMP6		"ICMP6"
 #define K_IN		"IN"
+#define K_PSI_IO	"IO"
 #define K_IP		"IP"
 #define K_IP6		"IP6"
+#define K_LOAD		"LOAD"
+#define K_PSI_MEM	"MEM"
 #define K_MOUNT		"MOUNT"
 #define K_NFS		"NFS"
 #define K_NFSD		"NFSD"
+#define K_PSI		"PSI"
 #define K_SID		"SID"
 #define K_SOCK		"SOCK"
 #define K_SOCK6		"SOCK6"
@@ -238,6 +246,7 @@
 #define K_CUSTOMCOL	"customcol"
 #define K_BWCOL		"bwcol"
 #define K_PCPARCHIVE	"pcparchive="
+#define K_HZ		"hz="
 
 /* Environment variables */
 #define ENV_COLORS_PALETTE	"S_COLORS_PALETTE"
@@ -813,10 +822,22 @@ struct record_header {
  * be counted, even if the activity is not collected.
  */
 #define AO_ALWAYS_COUNTED	0x200
+/*
+ * Indicate that corresponding activity should be collected only
+ * if a test has been successfully passed before.
+ * The test corresponds to the count() function (see @f_count_index).
+ * This is intended for activities with a fixed number of items but
+ * whose {/proc|/sys} files are not necessarily available, e.g. depending
+ * on the kernel version used. Unlike activities with AO_COUNTED flag,
+ * those having AO_DETECTED flag won't have a @has_nr number preceding
+ * their statistics structures in datafile.
+ */
+#define AO_DETECTED		0x400
 
 #define IS_COLLECTED(m)		(((m) & AO_COLLECTED)        == AO_COLLECTED)
 #define IS_SELECTED(m)		(((m) & AO_SELECTED)         == AO_SELECTED)
 #define HAS_COUNT_FUNCTION(m)	(((m) & AO_COUNTED)          == AO_COUNTED)
+#define HAS_DETECT_FUNCTION(m)	(((m) & AO_DETECTED)         == AO_DETECTED)
 #define HAS_PERSISTENT_VALUES(m) (((m) & AO_PERSISTENT)      == AO_PERSISTENT)
 #define CLOSE_MARKUP(m)		(((m) & AO_CLOSE_MARKUP)     == AO_CLOSE_MARKUP)
 #define HAS_MULTIPLE_OUTPUTS(m)	(((m) & AO_MULTIPLE_OUTPUTS) == AO_MULTIPLE_OUTPUTS)
@@ -868,7 +889,8 @@ struct activity {
 	unsigned int group;
 	/*
 	 * Index in f_count[] array to determine function used to count
-	 * the number of items (serial lines, network interfaces, etc.) -> @nr
+	 * the number of items (serial lines, network interfaces, etc.) for
+	 * activities with AO_COUNTED flag. Determine @nr value.
 	 * Such a function should _always_ return a value greater than
 	 * or equal to 0.
 	 *
@@ -878,6 +900,10 @@ struct activity {
 	 * These functions are called even if corresponding activities have not
 	 * been selected, to make sure that all items have been calculated
 	 * (including #CPU, etc.)
+	 *
+	 * The count() function may also be used to know if an activity (with
+	 * AO_DETECTED flag) can actually be collected based on the presence of
+	 * {/proc|/sys} files.
 	 */
 	int f_count_index;
 	/*
@@ -1334,6 +1360,8 @@ __nr_t wrap_get_filesystem_nr
 	(struct activity *);
 __nr_t wrap_get_fchost_nr
 	(struct activity *);
+__nr_t wrap_detect_psi
+	(struct activity *);
 
 /* Functions used to read activities statistics */
 __read_funct_t wrap_read_stat_cpu
@@ -1414,6 +1442,12 @@ __read_funct_t wrap_read_fchost
 	(struct activity *);
 __read_funct_t wrap_read_softnet
 	(struct activity *);
+__read_funct_t wrap_read_psicpu
+	(struct activity *);
+__read_funct_t wrap_read_psiio
+	(struct activity *);
+__read_funct_t wrap_read_psimem
+	(struct activity *);
 
 /* Other functions */
 int check_alt_sa_dir
@@ -1491,6 +1525,8 @@ int parse_sar_m_opt
 	(char * [], int *, struct activity * []);
 int parse_sar_n_opt
 	(char * [], int *, struct activity * []);
+int parse_sar_q_opt
+	(char * [], int *, struct activity * []);
 int parse_timestamp
 	(char * [], int *, struct tstamp *, const char *);
 void print_report_hdr
