@@ -1,6 +1,6 @@
 /*
  * sadc: system activity data collector
- * (C) 1999-2019 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2020 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -363,6 +363,19 @@ void sa_sys_init(void)
 			act[i]->options &= ~AO_COLLECTED;
 		}
 
+		if (HAS_DETECT_FUNCTION(act[i]->options) && IS_COLLECTED(act[i]->options)) {
+			idx = act[i]->f_count_index;
+
+			/* Detect if files needed by activity exist */
+			if (f_count_results[idx] < 0) {
+				f_count_results[idx] = (f_count[idx])(act[i]);
+			}
+			if (f_count_results[idx] == 0) {
+				/* Files not present */
+				act[i]->options &= ~AO_COLLECTED;
+			}
+		}
+
 		/* Set default activity list */
 		id_seq[i] = act[i]->id;
 	}
@@ -512,14 +525,14 @@ void setup_file_hdr(int fd)
 
 	/* Get system name, release number, hostname and machine architecture */
 	__uname(&header);
-	strncpy(file_hdr.sa_sysname, header.sysname, UTSNAME_LEN);
-	file_hdr.sa_sysname[UTSNAME_LEN - 1]  = '\0';
-	strncpy(file_hdr.sa_nodename, header.nodename, UTSNAME_LEN);
-	file_hdr.sa_nodename[UTSNAME_LEN - 1] = '\0';
-	strncpy(file_hdr.sa_release, header.release, UTSNAME_LEN);
-	file_hdr.sa_release[UTSNAME_LEN - 1]  = '\0';
-	strncpy(file_hdr.sa_machine, header.machine, UTSNAME_LEN);
-	file_hdr.sa_machine[UTSNAME_LEN - 1]  = '\0';
+	strncpy(file_hdr.sa_sysname, header.sysname, sizeof(file_hdr.sa_sysname));
+	file_hdr.sa_sysname[sizeof(file_hdr.sa_sysname) - 1]  = '\0';
+	strncpy(file_hdr.sa_nodename, header.nodename, sizeof(file_hdr.sa_nodename));
+	file_hdr.sa_nodename[sizeof(file_hdr.sa_nodename) - 1] = '\0';
+	strncpy(file_hdr.sa_release, header.release, sizeof(file_hdr.sa_release));
+	file_hdr.sa_release[sizeof(file_hdr.sa_release) - 1]  = '\0';
+	strncpy(file_hdr.sa_machine, header.machine, sizeof(file_hdr.sa_machine));
+	file_hdr.sa_machine[sizeof(file_hdr.sa_machine) - 1]  = '\0';
 
 	/* Get timezone value and save it */
 	tzset();
@@ -674,7 +687,7 @@ void write_stats(int ofd)
 			continue;
 
 		if (IS_COLLECTED(act[p]->options)) {
-			if (act[p]->f_count_index >= 0) {
+			if (HAS_COUNT_FUNCTION(act[p]->options) && (act[p]->f_count_index >= 0)) {
 				if (write_all(ofd, &(act[p]->_nr0), sizeof(__nr_t)) != sizeof(__nr_t)) {
 					p_write_error();
 				}
@@ -937,7 +950,7 @@ void open_ofile(int *ofd, char ofile[], int restart_mark)
 		}
 
 		if ((file_act[i].has_nr && (act[p]->f_count_index < 0)) ||
-		    (!file_act[i].has_nr && (act[p]->f_count_index >= 0))) {
+		    (!file_act[i].has_nr && (act[p]->f_count_index >= 0) && HAS_COUNT_FUNCTION(act[p]->options))) {
 #ifdef DEBUG
 			fprintf(stderr, "%s: %s: has_nr=%d count_index=%d\n",
 				__FUNCTION__, act[p]->name, file_act[i].has_nr, act[p]->f_count_index);
@@ -1153,8 +1166,8 @@ void rw_sa_stat_loop(long count, int stdfd, int ofd, char ofile[],
 		/* Rotate activity file if necessary */
 		if (WANT_SA_ROTAT(flags)) {
 			/* The user specified '-' as the filename to use */
-			strncpy(new_ofile, sa_dir, MAX_FILE_LEN - 1);
-			new_ofile[MAX_FILE_LEN - 1] = '\0';
+			strncpy(new_ofile, sa_dir, sizeof(new_ofile) - 1);
+			new_ofile[sizeof(new_ofile) - 1] = '\0';
 			set_default_file(new_ofile, 0, USE_SA_YYYYMMDD(flags));
 
 			if (strcmp(ofile, new_ofile)) {
@@ -1245,8 +1258,8 @@ int main(int argc, char **argv)
 			if (!argv[++opt]) {
 				usage(argv[0]);
 			}
-			strncpy(comment, argv[opt], MAX_COMMENT_LEN);
-			comment[MAX_COMMENT_LEN - 1] = '\0';
+			strncpy(comment, argv[opt], sizeof(comment));
+			comment[sizeof(comment) - 1] = '\0';
 			if (!strlen(comment)) {
 				usage(argv[0]);
 			}
@@ -1281,8 +1294,8 @@ int main(int argc, char **argv)
 			}
 			else {
 				/* Write data to file */
-				strncpy(ofile, argv[opt], MAX_FILE_LEN);
-				ofile[MAX_FILE_LEN - 1] = '\0';
+				strncpy(ofile, argv[opt], sizeof(ofile));
+				ofile[sizeof(ofile) - 1] = '\0';
 			}
 		}
 
