@@ -1,6 +1,6 @@
 /*
  * sadf: system activity data formatter
- * (C) 1999-2019 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2020 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -70,6 +70,7 @@ unsigned int dm_major;		/* Device-mapper major number */
 unsigned int format = 0;	/* Output format */
 unsigned int f_position = 0;	/* Output format position in array */
 unsigned int canvas_height = 0; /* SVG canvas height value set with option -O */
+unsigned int user_hz = 0;	/* HZ value set with option -O */
 
 /* File header */
 struct file_header file_hdr;
@@ -347,8 +348,8 @@ void list_fields(unsigned int act_id)
 			}
 			else {
 				msk = 1;
-				strncpy(hline, act[i]->hdr_line, HEADER_LINE_LEN - 1);
-				hline[HEADER_LINE_LEN - 1] = '\0';
+				strncpy(hline, act[i]->hdr_line, sizeof(hline) - 1);
+				hline[sizeof(hline) - 1] = '\0';
 				for (hl = strtok(hline, "|"); hl; hl = strtok(NULL, "|"), msk <<= 1) {
 					if ((hl != NULL) && ((act[i]->opt_flags & 0xff) & msk)) {
 						if (strchr(hl, '&')) {
@@ -1622,8 +1623,15 @@ int main(int argc, char **argv)
 				}
 				else if (!strncmp(t, K_PCPARCHIVE, strlen(K_PCPARCHIVE))) {
 					v = t + strlen(K_PCPARCHIVE);
-					strncpy(pcparchive, v, MAX_FILE_LEN);
-					pcparchive[MAX_FILE_LEN - 1] = '\0';
+					strncpy(pcparchive, v, sizeof(pcparchive));
+					pcparchive[sizeof(pcparchive) - 1] = '\0';
+				}
+				else if (!strncmp(t, K_HZ, strlen(K_HZ))) {
+					v = t + strlen(K_HZ);
+					if (!strlen(v) || (strspn(v, DIGITS) != strlen(v))) {
+						usage(argv[0]);
+					}
+					user_hz = atoi(v);
 				}
 				else {
 					usage(argv[0]);
@@ -1668,6 +1676,19 @@ int main(int argc, char **argv)
 			}
 		}
 
+		else if (!strcmp(argv[opt], "-q")) {
+			if (!sar_options) {
+				usage(argv[0]);
+			}
+			else if (!argv[++opt]) {
+				SELECT_ACTIVITY(A_QUEUE);
+			}
+			/* Parse option -q */
+			else if (parse_sar_q_opt(argv, &opt, act)) {
+				SELECT_ACTIVITY(A_QUEUE);
+			}
+		}
+
 		else if (!strncmp(argv[opt], "-", 1)) {
 			/* Other options not previously tested */
 			if (sar_options) {
@@ -1783,8 +1804,8 @@ int main(int argc, char **argv)
 				usage(argv[0]);
 			}
 			/* Write data to file */
-			strncpy(dfile, argv[opt++], MAX_FILE_LEN);
-			dfile[MAX_FILE_LEN - 1] = '\0';
+			strncpy(dfile, argv[opt++], sizeof(dfile));
+			dfile[sizeof(dfile) - 1] = '\0';
 			/* Check if this is an alternate directory for sa files */
 			check_alt_sa_dir(dfile, 0, -1);
 		}
