@@ -1,6 +1,6 @@
 /*
  * sar: report system activity
- * (C) 1999-2019 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2020 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -122,10 +122,11 @@ void usage(char *progname)
 	print_usage_title(stderr, progname);
 	fprintf(stderr, _("Options are:\n"
 			  "[ -A ] [ -B ] [ -b ] [ -C ] [ -D ] [ -d ] [ -F [ MOUNT ] ] [ -H ] [ -h ]\n"
-			  "[ -p ] [ -q ] [ -r [ ALL ] ] [ -S ] [ -t ] [ -u [ ALL ] ] [ -V ]\n"
+			  "[ -p ] [ -r [ ALL ] ] [ -S ] [ -t ] [ -u [ ALL ] ] [ -V ]\n"
 			  "[ -v ] [ -W ] [ -w ] [ -y ] [ -z ]\n"
 			  "[ -I { <int_list> | SUM | ALL } ] [ -P { <cpu_list> | ALL } ]\n"
 			  "[ -m { <keyword> [,...] | ALL } ] [ -n { <keyword> [,...] | ALL } ]\n"
+			  "[ -q [ <keyword> [,...] | ALL ] ]\n"
 			  "[ --dev=<dev_list> ] [ --fs=<fs_list> ] [ --iface=<iface_list> ]\n"
 			  "[ --dec={ 0 | 1 | 2 } ] [ --help ] [ --human ] [ --sadc ]\n"
 			  "[ -j { SID | ID | LABEL | PATH | UUID | ... } ]\n"
@@ -186,7 +187,13 @@ void display_help(char *progname)
 		 "\t\tUDP6\tUDP traffic\t(v6)\n"
 		 "\t\tFC\tFibre channel HBAs\n"
 		 "\t\tSOFT\tSoftware-based network processing\n"));
-	printf(_("\t-q\tQueue length and load average statistics [A_QUEUE]\n"));
+	printf(_("\t-q [ <keyword> [,...] | PSI | ALL ]\n"
+		 "\t\tSystem load and pressure-stall statistics\n"
+		 "\t\tKeywords are:\n"
+		 "\t\tLOAD\tQueue length and load average statistics [A_QUEUE]\n"
+		 "\t\tCPU\tPressure-stall CPU statistics [A_PSI_CPU]\n"
+		 "\t\tIO\tPressure-stall I/O statistics [A_PSI_IO]\n"
+		 "\t\tMEM\tPressure-stall memory statistics [A_PSI_MEM]\n"));
 	printf(_("\t-r [ ALL ]\n"
 		 "\t\tMemory utilization statistics [A_MEMORY]\n"));
 	printf(_("\t-S\tSwap space utilization statistics [A_MEMORY]\n"));
@@ -373,9 +380,9 @@ void write_stats_avg(int curr, int read_from_file, unsigned int act_id)
 	/* Interval value in 1/100th of a second */
 	itv = get_interval(record_hdr[2].uptime_cs, record_hdr[curr].uptime_cs);
 
-	strncpy(timestamp[curr], _("Average:"), TIMESTAMP_LEN);
-	timestamp[curr][TIMESTAMP_LEN - 1] = '\0';
-	memcpy(timestamp[!curr], timestamp[curr], TIMESTAMP_LEN);
+	strncpy(timestamp[curr], _("Average:"), sizeof(timestamp[curr]));
+	timestamp[curr][sizeof(timestamp[curr]) - 1] = '\0';
+	memcpy(timestamp[!curr], timestamp[curr], sizeof(timestamp[!curr]));
 
 	/* Test stdout */
 	TEST_STDOUT(STDOUT_FILENO);
@@ -570,10 +577,11 @@ void write_stats_startup(int curr)
  * @buffer	Buffer where data will be saved.
  *
  * RETURNS:
- * 1 if end of file has been reached, 0 otherwise.
+ * 0 if all the data have been successfully read.
+ * Otherwise, return the number of bytes left to be read.
  ***************************************************************************
  */
-int sa_read(void *buffer, size_t size)
+size_t sa_read(void *buffer, size_t size)
 {
 	ssize_t n;
 
@@ -585,7 +593,7 @@ int sa_read(void *buffer, size_t size)
 		}
 
 		if (!n)
-			return 1;	/* EOF */
+			return size;	/* EOF */
 
 		size -= n;
 		buffer = (char *) buffer + n;
@@ -894,6 +902,15 @@ void read_header_data(void)
 		fprintf(stderr, "%s: sysstat_magic=%x format_magic=%x version=%s\n",
 			__FUNCTION__, file_magic.sysstat_magic, file_magic.format_magic, version);
 #endif
+		if (rc == FILE_MAGIC_SIZE) {
+			/*
+			 * No data (0 byte) have been sent by sadc.
+			 * This is probably because no activities have been collected
+			 * ("Requested activities not available"). In this case, don't
+			 * display an error message: Exit now.
+			 */
+			exit(3);
+		}
 		print_read_error(INCONSISTENT_INPUT_DATA);
 	}
 
@@ -1361,8 +1378,8 @@ int main(int argc, char **argv)
 			/* Save stats to a file */
 			if ((argv[++opt]) && strncmp(argv[opt], "-", 1) &&
 			    (strspn(argv[opt], DIGITS) != strlen(argv[opt]))) {
-				strncpy(to_file, argv[opt++], MAX_FILE_LEN);
-				to_file[MAX_FILE_LEN - 1] = '\0';
+				strncpy(to_file, argv[opt++], sizeof(to_file));
+				to_file[sizeof(to_file) - 1] = '\0';
 			}
 			else {
 				strcpy(to_file, "-");
@@ -1377,8 +1394,8 @@ int main(int argc, char **argv)
 			/* Read stats from a file */
 			if ((argv[++opt]) && strncmp(argv[opt], "-", 1) &&
 			    (strspn(argv[opt], DIGITS) != strlen(argv[opt]))) {
-				strncpy(from_file, argv[opt++], MAX_FILE_LEN);
-				from_file[MAX_FILE_LEN - 1] = '\0';
+				strncpy(from_file, argv[opt++], sizeof(from_file));
+				from_file[sizeof(from_file) - 1] = '\0';
 				/* Check if this is an alternate directory for sa files */
 				check_alt_sa_dir(from_file, day_offset, -1);
 			}
@@ -1432,6 +1449,16 @@ int main(int argc, char **argv)
 			}
 		}
 
+		else if (!strcmp(argv[opt], "-q")) {
+			if (!argv[++opt]) {
+				SELECT_ACTIVITY(A_QUEUE);
+			}
+			/* Parse option -q */
+			else if (parse_sar_q_opt(argv, &opt, act)) {
+				SELECT_ACTIVITY(A_QUEUE);
+			}
+		}
+
 #ifdef TEST
 		else if (!strncmp(argv[opt], "--getenv", 8)) {
 			__env = TRUE;
