@@ -1,6 +1,6 @@
 /*
  * xml_stats.c: Funtions used by sadf to display statistics in XML.
- * (C) 1999-2019 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2020 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -91,6 +91,33 @@ void xml_markup_power_management(int tab, int action)
 	}
 }
 
+/*
+ ***************************************************************************
+ * Open or close <psi> markup.
+ *
+ * IN:
+ * @tab		Number of tabulations.
+ * @action	Open or close action.
+ ***************************************************************************
+ */
+void xml_markup_psi(int tab, int action)
+{
+	static int markup_state = CLOSE_XML_MARKUP;
+
+	if (action == markup_state)
+		return;
+	markup_state = action;
+
+	if (action == OPEN_XML_MARKUP) {
+		/* Open markup */
+		xprintf(tab, "<psi per=\"second\">");
+	}
+	else {
+		/* Close markup */
+		xprintf(tab, "</psi>");
+	}
+}
+
 /*
  ***************************************************************************
  * Display CPU statistics in XML.
@@ -462,7 +489,7 @@ __print_funct_t xml_print_memory_stats(struct activity *a, int curr, int tab,
 		*smc = (struct stats_memory *) a->buf[curr];
 	unsigned long long nousedmem;
 
-	xprintf(tab, "<memory per=\"second\" unit=\"kB\">");
+	xprintf(tab, "<memory unit=\"kB\">");
 
 	if (DISPLAY_MEMORY(a->opt_flags)) {
 
@@ -784,7 +811,7 @@ __print_funct_t xml_print_net_dev_stats(struct activity *a, int curr, int tab,
 	struct stats_net_dev *sndc, *sndp, sndzero;
 	double rxkb, txkb, ifutil;
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	memset(&sndzero, 0, STATS_NET_DEV_SIZE);
@@ -860,7 +887,7 @@ __print_funct_t xml_print_net_edev_stats(struct activity *a, int curr, int tab,
 	int i, j;
 	struct stats_net_edev *snedc, *snedp, snedzero;
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	memset(&snedzero, 0, STATS_NET_EDEV_SIZE);
@@ -935,7 +962,7 @@ __print_funct_t xml_print_net_nfs_stats(struct activity *a, int curr, int tab,
 		*snnc = (struct stats_net_nfs *) a->buf[curr],
 		*snnp = (struct stats_net_nfs *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -980,7 +1007,7 @@ __print_funct_t xml_print_net_nfsd_stats(struct activity *a, int curr, int tab,
 		*snndc = (struct stats_net_nfsd *) a->buf[curr],
 		*snndp = (struct stats_net_nfsd *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -1034,7 +1061,7 @@ __print_funct_t xml_print_net_sock_stats(struct activity *a, int curr, int tab,
 	struct stats_net_sock
 		*snsc = (struct stats_net_sock *) a->buf[curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -1079,7 +1106,7 @@ __print_funct_t xml_print_net_ip_stats(struct activity *a, int curr, int tab,
 		*snic = (struct stats_net_ip *) a->buf[curr],
 		*snip = (struct stats_net_ip *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -1128,7 +1155,7 @@ __print_funct_t xml_print_net_eip_stats(struct activity *a, int curr, int tab,
 		*sneic = (struct stats_net_eip *) a->buf[curr],
 		*sneip = (struct stats_net_eip *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -1177,7 +1204,7 @@ __print_funct_t xml_print_net_icmp_stats(struct activity *a, int curr, int tab,
 		*snic = (struct stats_net_icmp *) a->buf[curr],
 		*snip = (struct stats_net_icmp *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -1238,7 +1265,7 @@ __print_funct_t xml_print_net_eicmp_stats(struct activity *a, int curr, int tab,
 		*sneic = (struct stats_net_eicmp *) a->buf[curr],
 		*sneip = (struct stats_net_eicmp *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -1295,7 +1322,7 @@ __print_funct_t xml_print_net_tcp_stats(struct activity *a, int curr, int tab,
 		*sntc = (struct stats_net_tcp *) a->buf[curr],
 		*sntp = (struct stats_net_tcp *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -1336,7 +1363,7 @@ __print_funct_t xml_print_net_etcp_stats(struct activity *a, int curr, int tab,
 		*snetc = (struct stats_net_etcp *) a->buf[curr],
 		*snetp = (struct stats_net_etcp *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -1379,7 +1406,7 @@ __print_funct_t xml_print_net_udp_stats(struct activity *a, int curr, int tab,
 		*snuc = (struct stats_net_udp *) a->buf[curr],
 		*snup = (struct stats_net_udp *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -1419,7 +1446,7 @@ __print_funct_t xml_print_net_sock6_stats(struct activity *a, int curr, int tab,
 	struct stats_net_sock6
 		*snsc = (struct stats_net_sock6 *) a->buf[curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -1460,7 +1487,7 @@ __print_funct_t xml_print_net_ip6_stats(struct activity *a, int curr, int tab,
 		*snic = (struct stats_net_ip6 *) a->buf[curr],
 		*snip = (struct stats_net_ip6 *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -1513,7 +1540,7 @@ __print_funct_t xml_print_net_eip6_stats(struct activity *a, int curr, int tab,
 		*sneic = (struct stats_net_eip6 *) a->buf[curr],
 		*sneip = (struct stats_net_eip6 *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -1568,7 +1595,7 @@ __print_funct_t xml_print_net_icmp6_stats(struct activity *a, int curr, int tab,
 		*snic = (struct stats_net_icmp6 *) a->buf[curr],
 		*snip = (struct stats_net_icmp6 *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -1635,7 +1662,7 @@ __print_funct_t xml_print_net_eicmp6_stats(struct activity *a, int curr, int tab
 		*sneic = (struct stats_net_eicmp6 *) a->buf[curr],
 		*sneip = (struct stats_net_eicmp6 *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -1690,7 +1717,7 @@ __print_funct_t xml_print_net_udp6_stats(struct activity *a, int curr, int tab,
 		*snuc = (struct stats_net_udp6 *) a->buf[curr],
 		*snup = (struct stats_net_udp6 *) a->buf[!curr];
 
-	if (!IS_SELECTED(a->options) || (a->nr <= 0))
+	if (!IS_SELECTED(a->options) || (a->nr[curr] <= 0))
 		goto close_xml_markup;
 
 	xml_markup_network(tab, OPEN_XML_MARKUP);
@@ -2276,3 +2303,142 @@ close_xml_markup:
 		xml_markup_network(tab, CLOSE_XML_MARKUP);
 	}
 }
+
+/*
+ ***************************************************************************
+ * Display pressure-stall CPU statistics in XML.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @curr	Index in array for current sample statistics.
+ * @tab		Indentation in XML output.
+ * @itv		Interval of time in 1/100th of a second.
+ ***************************************************************************
+ */
+__print_funct_t xml_print_psicpu_stats(struct activity *a, int curr, int tab,
+				       unsigned long long itv)
+{
+	struct stats_psi_cpu
+		*psic = (struct stats_psi_cpu *) a->buf[curr],
+		*psip = (struct stats_psi_cpu *) a->buf[!curr];
+
+	if (!IS_SELECTED(a->options))
+		goto close_xml_markup;
+
+	xml_markup_psi(tab, OPEN_XML_MARKUP);
+	tab++;
+
+	xprintf(tab, "<psi-cpu "
+		"some_avg10=\"%.2f\" "
+		"some_avg60=\"%.2f\" "
+		"some_avg300=\"%.2f\" "
+		"some_avg=\"%.2f\"/>",
+		(double) psic->some_acpu_10  / 100,
+		(double) psic->some_acpu_60  / 100,
+		(double) psic->some_acpu_300 / 100,
+		((double) psic->some_cpu_total - psip->some_cpu_total) / (100 * itv));
+	tab--;
+
+close_xml_markup:
+	if (CLOSE_MARKUP(a->options)) {
+		xml_markup_psi(tab, CLOSE_XML_MARKUP);
+	}
+}
+
+/*
+ ***************************************************************************
+ * Display pressure-stall I/O statistics in XML.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @curr	Index in array for current sample statistics.
+ * @tab		Indentation in XML output.
+ * @itv		Interval of time in 1/100th of a second.
+ ***************************************************************************
+ */
+__print_funct_t xml_print_psiio_stats(struct activity *a, int curr, int tab,
+				      unsigned long long itv)
+{
+	struct stats_psi_io
+		*psic = (struct stats_psi_io *) a->buf[curr],
+		*psip = (struct stats_psi_io *) a->buf[!curr];
+
+	if (!IS_SELECTED(a->options))
+		goto close_xml_markup;
+
+	xml_markup_psi(tab, OPEN_XML_MARKUP);
+	tab++;
+
+	xprintf(tab, "<psi-io "
+		"some_avg10=\"%.2f\" "
+		"some_avg60=\"%.2f\" "
+		"some_avg300=\"%.2f\" "
+		"some_avg=\"%.2f\" "
+		"full_avg10=\"%.2f\" "
+		"full_avg60=\"%.2f\" "
+		"full_avg300=\"%.2f\" "
+		"full_avg=\"%.2f\"/>",
+		(double) psic->some_aio_10  / 100,
+		(double) psic->some_aio_60  / 100,
+		(double) psic->some_aio_300 / 100,
+		((double) psic->some_io_total - psip->some_io_total) / (100 * itv),
+		(double) psic->full_aio_10  / 100,
+		(double) psic->full_aio_60  / 100,
+		(double) psic->full_aio_300 / 100,
+		((double) psic->full_io_total - psip->full_io_total) / (100 * itv));
+	tab--;
+
+close_xml_markup:
+	if (CLOSE_MARKUP(a->options)) {
+		xml_markup_psi(tab, CLOSE_XML_MARKUP);
+	}
+}
+
+/*
+ ***************************************************************************
+ * Display pressure-stall memory statistics in XML.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @curr	Index in array for current sample statistics.
+ * @tab		Indentation in XML output.
+ * @itv		Interval of time in 1/100th of a second.
+ ***************************************************************************
+ */
+__print_funct_t xml_print_psimem_stats(struct activity *a, int curr, int tab,
+				       unsigned long long itv)
+{
+	struct stats_psi_mem
+		*psic = (struct stats_psi_mem *) a->buf[curr],
+		*psip = (struct stats_psi_mem *) a->buf[!curr];
+
+	if (!IS_SELECTED(a->options))
+		goto close_xml_markup;
+
+	xml_markup_psi(tab, OPEN_XML_MARKUP);
+	tab++;
+
+	xprintf(tab, "<psi-mem "
+		"some_avg10=\"%.2f\" "
+		"some_avg60=\"%.2f\" "
+		"some_avg300=\"%.2f\" "
+		"some_avg=\"%.2f\" "
+		"full_avg10=\"%.2f\" "
+		"full_avg60=\"%.2f\" "
+		"full_avg300=\"%.2f\" "
+		"full_avg=\"%.2f\"/>",
+		(double) psic->some_amem_10  / 100,
+		(double) psic->some_amem_60  / 100,
+		(double) psic->some_amem_300 / 100,
+		((double) psic->some_mem_total - psip->some_mem_total) / (100 * itv),
+		(double) psic->full_amem_10  / 100,
+		(double) psic->full_amem_60  / 100,
+		(double) psic->full_amem_300 / 100,
+		((double) psic->full_mem_total - psip->full_mem_total) / (100 * itv));
+	tab--;
+
+close_xml_markup:
+	if (CLOSE_MARKUP(a->options)) {
+		xml_markup_psi(tab, CLOSE_XML_MARKUP);
+	}
+}
